using System.Linq;
using NHibernate.Linq;
using NUnit.Framework;

namespace NHibernate.Test.NHSpecificTest.NH2906 {
   public class Fixture : BugTestCase {
      private const string Entity1Name = "Name of Entity1";
      private const string Entity2Category = "Category of Entity2";

      protected override void OnSetUp() {
         using (ISession session = OpenSession()) {
            using (ITransaction tx = session.BeginTransaction()) {
               var entity1 = new Entity1 { Name = Entity1Name };
               var entity2 = new Entity2 { Category = Entity2Category };
               entity1.Entity2s.Add(entity2);
               entity2.Entity1 = entity1;

               session.Save(entity1);
               session.Save(entity2);

               tx.Commit();
            }
         }
      }

      protected override void OnTearDown() {
         base.OnTearDown();

         using (ISession session = OpenSession()) {
            using (ITransaction tx = session.BeginTransaction()) {
               session.Delete("from Entity2");
               session.Delete("from Entity1");
               tx.Commit();
            }
         }
      }

      [Test]
      public void SameParameterName_TwoDifferentlyTypedConditions() {
         using (ISession session = OpenSession()) {
            using (ITransaction tx = session.BeginTransaction()) {
               // Changing the "x" in one of the "Where" calls to something different makes the test pass.
               var result = session.Query<Entity1>()
                  .Where(x => x.Name == Entity1Name)
                  .SelectMany(x => x.Entity2s)
                  .Where(x => x.Category == Entity2Category)
                  .ToList();

               tx.Commit();

               Assert.AreEqual(1, result.Count);
               Assert.AreEqual(Entity2Category, result[0].Category);
               Assert.AreEqual(Entity1Name, result[0].Entity1.Name);
            }
         }
      }
   }
}